SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

Object subclass: #Cart
	instanceVariableNames: 'products catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'message handling' stamp: 'PaneroGomez 6/16/2012 16:31'!
messageTextGivenBookDontBelongToTheCatalog
	
	^ 'The given product doesnt belong to the catalog'.! !

!Cart class methodsFor: 'message handling' stamp: 'ExeNico 6/13/2012 21:35'!
messageTextNegativeQuantityError

	^ 'Quantity must be greater than zero'! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'ExeNico 6/13/2012 21:44'!
newWithCatalog: aCatalog

	^ self new initializeWith: aCatalog.! !

!Cart methodsFor: 'updating' stamp: 'ExeNico 6/13/2012 21:45'!
addProduct: aProduct
 	
	self addProduct: aProduct quantity: 1.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:41'!
addProduct: aProduct quantity: anAmount 
 	
	(anAmount <= 0)
		ifTrue: [
			self error: self class messageTextNegativeQuantityError.
		].
	
	(catalog includes: aProduct)
		ifFalse: [
			self error: self class messageTextGivenBookDontBelongToTheCatalog.		
		].
	
	products add: aProduct withOccurrences: anAmount. ! !

!Cart methodsFor: 'comparing' stamp: 'ExequielGomez 6/17/2012 19:14'!
catalog

	^ catalog copy.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:18'!
cleanCurrentProducts

	products removeAll.! !

!Cart methodsFor: 'comparing' stamp: 'ExeNico 6/13/2012 21:43'!
initializeWith: aCatalog

	catalog := aCatalog.
	products := Bag new.! !

!Cart methodsFor: 'comparing' stamp: 'PaneroGomez 6/11/2012 23:41'!
isEmpty

	^ products isEmpty.! !

!Cart methodsFor: 'comparing' stamp: 'ExeNico 6/13/2012 21:38'!
products

	^ products copy.! !

!Cart methodsFor: 'accessing' stamp: 'PaneroGomez 6/8/2012 20:13'!
quantityOf: aProduct
   
    ^ self products occurrencesOf: aProduct.! !

!Cart methodsFor: 'comparing' stamp: 'ExeNico 6/13/2012 21:39'!
size

	^ products size.! !

Object subclass: #Cashier
	instanceVariableNames: 'merchantProcessor amountOfTransactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'error handling' stamp: 'PaneroGomez 6/16/2012 17:34'!
messageTextTheCartMustNotBeEmpty
	
	^ 'The Cart should have any products in order to check out'.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/17/2012 22:27'!
amountOfTransaction

	^ amountOfTransactions.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:58'!
checkOutWith: aCart usingCreditCard: aCreditCard

	|total products|

	(aCart size  <= 0)
		ifTrue: [
			self error: self class messageTextTheCartMustNotBeEmpty.
		].
	
	total := 0.

	products := aCart products.
	
	products do:[ :product |
			total := total + (product price).
		].
		
	(self collect: total usingMerchantProcessorFor: aCart usingCreditCard: (aCreditCard number) validUntil: (aCreditCard validUntil) fromTheOwner: (aCreditCard ownerName))
		ifFalse: [
				self error: self class messageTextMerchantProcessorFaild.
			].
	
	aCart cleanCurrentProducts.
	
	^ total.
		
	
	
	! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/17/2012 22:29'!
collect: aSmallInteger usingMerchantProcessorFor: aCart usingCreditCard: aByteString validUntil: aMonth fromTheOwner: aByteString5 

	amountOfTransactions := amountOfTransactions + 1.
	^true! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/17/2012 22:29'!
initialize

	amountOfTransactions := 0.! !

Object subclass: #CreditCard
	instanceVariableNames: 'number validUntil ownerName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CreditCard class methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/17/2012 23:41'!
messageTextTheCreditCardHasExpired
	
	^ 'The Credit Card has expired'.! !

!CreditCard class methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/17/2012 23:41'!
messageTextTheOwnerNameIsToLong
	
	^ 'The Name of the credti card owner is longer than 30 characters'.! !

!CreditCard class methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:22'!
newWith: aNumber andWithExpirationDate: expirationMonth andAOwnerName: aOwnerName
	
	|currentDate|
	
	currentDate := Month current.
	
	(expirationMonth < currentDate)
		ifTrue: [
			self error: self messageTextTheCreditCardHasExpired.
		].
	
	(aOwnerName size > 30)
		ifTrue: [
			self error: self messageTextTheOwnerNameIsToLong.
		].

	^ self new initializeWithNumber: aNumber andWithExpirationDate: aOwnerName andAOwnerName: aOwnerName.! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:22'!
initializeWithNumber: aNumber andWithExpirationDate: aMonth andAOwnerName: aOwnerName

	number := aNumber.
	validUntil := aMonth.
	ownerName := aOwnerName.! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:23'!
number

	^ number copy.! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:23'!
ownerName

	^ ownerName copy.! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:23'!
validUntil

	^ validUntil copy.! !

Object subclass: #Product
	instanceVariableNames: 'itemID price'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

Product subclass: #EmptyProduct
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!EmptyProduct class methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:53'!
new

	^ self new initializeWithAnID: '' andAPrice: 0. ! !

!Product class methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:29'!
newWithAnID: anID andAPrice: aPrice

	^ self new initializeWithAnID: anID andAPrice: aPrice.! !

!Product methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:49'!
= otherProduct

	(otherProduct isKindOf: self class)
		ifFalse: [^ false].
	
	^ ( itemID = otherProduct itemID and: [price = otherProduct price] ).! !

!Product methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:43'!
hash

	^ itemID hash + price hash.! !

!Product methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:30'!
initializeWithAnID: anID andAPrice: aPrice

	itemID := anID.
	price := aPrice.! !

!Product methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:31'!
itemID

	^ itemID copy.! !

!Product methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:31'!
price

	^ price copy.! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:35'!
aCatalogEmpty

	^ Set new .! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:35'!
aCatalogNotEmpty
	
	|aCatalog aProduct1 aProduct2 aProduct3 aProduct4|
	 aCatalog := self aCatalogEmpty .
	
	aProduct1 := Product newWithAnID: 'isbn12234-14252-15252-152512' andAPrice: 12.
	aProduct2 := Product newWithAnID: 'isbn12234-14252-15252-152513' andAPrice: 1.
	aProduct3 := Product newWithAnID: 'isbn12234-14252-15252-152514' andAPrice: 21.
	aProduct4 := Product newWithAnID: 'isbn12234-14252-15252-152515' andAPrice: 22.
		
	"TENDRIAMOS QUE REIFICAR LA MONDA"
	aCatalog add: aProduct1.
	aCatalog add: aProduct2.
	aCatalog add: aProduct3.
	aCatalog add: aProduct4.
	
	^ aCatalog ! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:31'!
aProductInTheCatalog

	^ Product newWithAnID: 'isbn12234-14252-15252-152512' andAPrice: 12.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:32'!
aProductNotInTheCatalog

	^ Product newWithAnID: 'isbn12234-14252-15252-152555' andAPrice: 34.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'PaneroGomez 6/18/2012 00:32'!
otherProductInTheCatalog

	^ Product newWithAnID: 'isbn12234-14252-15252-152514' andAPrice: 21.! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/16/2012 16:10'!
test01CartShouldBeEmptyWhenCreated

	| aCart |
	
	aCart := Cart newWithCatalog: (self aCatalogNotEmpty).
	
	self assert: ( aCart isEmpty ).! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:36'!
test02CartShouldNotBeEmptyWhenAddAnElement

    | aCart aProduct|

    aCart := Cart newWithCatalog: self aCatalogNotEmpty.
    aProduct := self aProductInTheCatalog.
    aCart addProduct: aProduct.
   
    self deny: aCart isEmpty! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:36'!
test03TwoEqualProductsCanBeAddedToTheCart

	|aCart aProduct otherProduct aCatalog|
	
	aCatalog := self aCatalogNotEmpty.
	
	aProduct  := self aProductInTheCatalog .
	otherProduct  := aProduct copy.
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart addProduct: aProduct .
	aCart addProduct: otherProduct .

	self assert: ( aProduct = otherProduct  and: [aCart size = 2]).! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:37'!
test04WhenAskAProductQuantityOfADontAddedProductShouldReturnZero
  
	|aCart aProduct otherProduct aCatalog|
	
	aCatalog := self aCatalogNotEmpty.
	
	aProduct  := self aProductInTheCatalog .
	otherProduct  := self otherProductInTheCatalog .
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart addProduct: aProduct .

	self assert: ( (aCart quantityOf: otherProduct) = 0).! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:37'!
test05WhenAskForAProductQuantityShouldReturnTheQuantityAdded
  
	|aCart aProduct otherProduct aCatalog|
	
	aCatalog := self aCatalogNotEmpty.
	
	aProduct  := self aProductInTheCatalog .
	otherProduct  := aProduct copy.
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart addProduct: aProduct .
	aCart addProduct: otherProduct .

	self assert: ( (aCart quantityOf: aProduct) = 2).
	self assert: ( (aCart quantityOf: otherProduct) = 2).! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:38'!
test06WhenListACartShouldHaveAllAddedElements

	|aCart aProduct otherProduct productsAdded aCatalog|
	
	aCatalog := self aCatalogNotEmpty.
	
	aProduct  := self aProductInTheCatalog .
	otherProduct  := self otherProductInTheCatalog .
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart addProduct: aProduct .
	aCart addProduct: otherProduct .

	productsAdded := (Array with: aProduct with: otherProduct ) .

	(aCart products) do: [ :aProductList | self assert: (productsAdded includes: aProductList)].! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:38'!
test07AddingAProductSpecifingTheQuantityShouldAddTheRightAmountOfProducts

	|aCart aProduct aCatalog|
	
	aCatalog := self aCatalogNotEmpty.
	aProduct  := self aProductInTheCatalog .
	
	aCart := Cart newWithCatalog: aCatalog.
	aCart addProduct: aProduct quantity: 3.

	self assert: ( aCart products size = 3).! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:38'!
test08AddingAProductSpecifingANegativeQuantityShouldRaiseAnError
	
	|aCart aProduct aCatalog|
	
	aCatalog := self aCatalogNotEmpty.
	aProduct  := self aProductInTheCatalog .
	
	aCart := Cart newWithCatalog: aCatalog.
	
	self 
		should: [aCart addProduct: aProduct quantity: -3.	]	 
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = Cart messageTextNegativeQuantityError) ].
		
	self assert: (aCart size = 0).! !

!TusLibrosTest methodsFor: 'cartTests' stamp: 'PaneroGomez 6/18/2012 00:39'!
test09AddingAProducThatIsNotInTheCatalogShouldRaiseAnError
	
	|aCart  otherProduct aCatalog|
	
	otherProduct  := self aProductNotInTheCatalog.
	
	aCatalog := self aCatalogEmpty.
	
	aCart := Cart newWithCatalog: aCatalog.
	
	self 
		should: [aCart addProduct: otherProduct.	]	 
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = Cart messageTextGivenBookDontBelongToTheCatalog) ].
		
	self assert: (aCart size = 0).! !

!TusLibrosTest methodsFor: 'cashierTests' stamp: 'PaneroGomez 6/18/2012 00:05'!
test10GivenAnEmptyCartWhenTryingToCheckOutShouldRaiseAnException

	| aCart aCashier aCreditCard|
	
	aCart := Cart newWithCatalog: (self aCatalogEmpty).
	aCashier := Cashier new.
	aCreditCard := CreditCard newWith: 123456789 andWithExpirationDate: Month current andAOwnerName: 'Pepe'.

	self 
		should: [ aCashier checkOutWith: aCart usingCreditCard: aCreditCard ]	 
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = Cashier messageTextTheCartMustNotBeEmpty) ].! !

!TusLibrosTest methodsFor: 'creditCardTests' stamp: 'PaneroGomez 6/18/2012 00:00'!
test11CreditCardsShouldNotBeExpired

	| expiredDateWithAnOlderYear expiredDateWithAnOlderMonth expiredDateWithAnOlderYearAndMonth|
	
	expiredDateWithAnOlderYear := Month month: 8 year: 2011.
	expiredDateWithAnOlderMonth := Month month: 1 year: 2012.
	expiredDateWithAnOlderYearAndMonth := Month month: 1 year: 2011.
	
	self 
		should: [ CreditCard newWith: 123456789 andWithExpirationDate: expiredDateWithAnOlderYear andAOwnerName: 'Pepe' ]	
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = CreditCard messageTextTheCreditCardHasExpired) ].
		
	self 
		should: [ CreditCard newWith: 123456789 andWithExpirationDate: expiredDateWithAnOlderMonth andAOwnerName: 'Pepe' ]	
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = CreditCard messageTextTheCreditCardHasExpired) ].
		
	self 
		should: [ CreditCard newWith: 123456789 andWithExpirationDate: expiredDateWithAnOlderYearAndMonth andAOwnerName: 'Pepe' ]	
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = CreditCard messageTextTheCreditCardHasExpired) ].! !

!TusLibrosTest methodsFor: 'creditCardTests' stamp: 'PaneroGomez 6/18/2012 00:03'!
test12CreditCardsShouldHaveAnOwnerNameNoLongerThan30Characters

	self 
		should: [ CreditCard newWith: 123456789 andWithExpirationDate: (Month current) andAOwnerName: 'Abcdefghijklmnopqrstuvwxyz Abcdefghijklmnopqrstuvwxyz'. ]	
		raise:  Error 
		withExceptionDo: [:anError | self assert: (anError messageText = CreditCard messageTextTheOwnerNameIsToLong) ].! !

!TusLibrosTest methodsFor: 'cashierTests' stamp: 'PaneroGomez 6/18/2012 00:54'!
test13GivenAnNonEmptyCartWhenTryingToCheckOutShouldCollectTheCorrectMoney

	| aCart aCashier aCatalog aProduct aCreditCard|
	
	aCatalog := self aCatalogNotEmpty.
	aCart := Cart newWithCatalog: (aCatalog ).
	
	aProduct := self aProductInTheCatalog .
	
	aCart addProduct: aProduct  quantity: 2.
	
	aCashier := Cashier new.
	
	aCreditCard := CreditCard newWith: 123456789 andWithExpirationDate: (Month month: 8 year: 2013) andAOwnerName: 'Pepe'.
	
	self assert: (2 * (aCatalog like: aProduct ifAbsent: [EmptyProduct new]) price = (aCashier checkOutWith: aCart usingCreditCard: aCreditCard)).
	
! !

!TusLibrosTest methodsFor: 'cashierTests' stamp: 'PaneroGomez 6/18/2012 00:13'!
test14AFailedCheckOutShouldNotMakeATransactionWithTheMerchantProcesor

	| aCashier transactionWithMerchantProcesor aCreditCard anEmptyCart |
	
	anEmptyCart := Cart newWithCatalog: (self aCatalogEmpty).

	aCashier := Cashier new.
	transactionWithMerchantProcesor := aCashier amountOfTransaction.
		
	aCreditCard := CreditCard newWith: 123456789 andWithExpirationDate: (Month current) andAOwnerName: 'Pepe'.
		
	" Transacion que falla por carrito vacio "
	self 
		should: [ aCashier checkOutWith: anEmptyCart usingCreditCard: aCreditCard ]	 
		raise:  Error 
		withExceptionDo: [:anError | self assert: (transactionWithMerchantProcesor = aCashier amountOfTransaction) ].
	
! !

!TusLibrosTest methodsFor: 'cashierTests' stamp: 'PaneroGomez 6/18/2012 00:40'!
test15ANonFailedCheckOutShouldMakeATransactionWithTheMerchantProcesor

	| aCart aCashier aCatalog aProduct aCreditCard |
	
	aCatalog := self aCatalogNotEmpty.
	aCart := Cart newWithCatalog: (aCatalog ).
	
	aProduct := self aProductInTheCatalog .
	
	aCart addProduct: aProduct  quantity: 2.
	
	aCashier := Cashier new.
	
	aCreditCard := CreditCard newWith: 123456789 andWithExpirationDate: (Month month: 8 year: 2013) andAOwnerName: 'Pepe'.
	
	aCashier checkOutWith: aCart usingCreditCard: aCreditCard.
	
	self assert: (1 = aCashier amountOfTransaction).
	! !
